// JNMM4LPXTDIOFunctions.c : Defines the entry point for the console application.
//

#include<stdio.h>
#include<stdlib.h>
#include<pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <sys/select.h>
#include <termios.h>
#include <stropts.h>
#include <unistd.h>

void dscsleep(int ms)
{
	usleep(ms*1000);
}
#define BYTE unsigned char
static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}


#include "../CANLib/libcan.h"

#define MAX_CAN 4
int fd[MAX_CAN];

unsigned int diobase_address = 0x340;
CANCONFIG canconfig;

//DIO Port No
#define DIOPORT_A 0
#define DIOPORT_B 1

#define DE_NONE 0


#define READ_BYTE					   1
#define READ_BYTE_FROM_ALL_PORTS	   2
#define READ_ALL_BITS				   3
#define WRITE_BYTE					   4
#define WRITE_BIT					   5
#define LOOP_BACK_TEST				   6


char dio_buffer[20];

void readByteDIOPort();
void readByteAllDIOPorts();
void readAllBitsDIOPort();
void writeByteDIOPort();
void writeBitDIOPort();
void DIOPortLoopbackTest();

int main()
{
	
	int can_fd;
	char input_buffer[20];
	char dio_buffer[20];
	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
	int flag;
	printf ( "DSCJNMMCAN4LP-XT_DEMO DIO Application \n" );


	can_fd = JNMMCANOpen(&flag);
	if(can_fd == -1)
	{
		return 0;
	}
	if(flag==isa_mode)
	{
	  	printf("\nDSC-JNMM-CAN4LP-XT-ISA Device\n");	  	
		printf("Enter the DIObase address (0x0100,0x0140,0x0180,0x01C0,0x0200,0x0240,0x0300,default= 0x0340 , hit ENTER) : ");
		fgets(dio_buffer, 20, stdin);

		if (dio_buffer[0] == '\n')
		{
			printf("DIOBase address default with 0x0340  \n");
		}
		else
		{
			sscanf(dio_buffer, "%x", &diobase_address);
		}
		JNMMCANDIOAddrInit(diobase_address);

	}
	else
	  	printf("\n DSC-JNMM-CAN4LP-XT-PCI Device\n");
	
	//		DIO INPUT AND OUTPUT
	//
	//		Perform a set of byte-level inputs and outputs. We start by sending
	//		an output byte value of 0 to the selected digital I/O port and then
	//		read from that same port again to verify that the correct value was
	//		exchanged. For word and bit-level operations, the steps are nearly
	//		identical with a few minor changes to function names and the size
	//		of the data to be transferred.
	//
	//		NOTE: Remember that the bytes received from port are supposed to be
	//			  equal to the inversion of the bytes sent. That is, the input
	//			  bytes are equal to 255 minus the output bytes.
	//
	//		Guide:
	//
	//		1. Read a Byte from  port 
	//		2. Read a Byte from all ports
	//		3. Read all bits from a  port
	//		4. Write a Byte to port
	//		5. Write a Bit to a port
	//		6. Port loopback with 0-255 read/write repetitive\n
	//		q. Quit Program
	// 		Choose these options in any order from the menu!
	//=========================================================================
	printf( "\nDIO INPUT AND OUTPUT: Main Menu \n" );

	do
	{
		intBuff = 0;
		printf (  "########################################################\n" );
		printf (  "1) Read a Byte from  port\n" );
		printf (  "2) Read a Byte from all ports\n" );
		printf (  "3) Read all bits from a  port\n" );
		printf (  "4) Write a Byte to port\n" );
		printf (  "5) Write a Bit to a port\n" );
		printf (  "6) Port loopback with 0-255 read/write repetitive\n" );
		printf (  "q) Quit Program \n" );
		printf (  "########################################################\n" );
		
		fgets ( input_buffer, 20, stdin );
        sscanf ( input_buffer, "%d", &intBuff );
        if ( input_buffer[0] == 'q' )
        {
            intBuff = 'q';
        }
		switch ( intBuff )
		{

			case READ_BYTE:
					readByteDIOPort();
					break;

				case READ_BYTE_FROM_ALL_PORTS:
	     			readByteAllDIOPorts();
					break;

				case READ_ALL_BITS:
					 readAllBitsDIOPort();
					 break;

				case WRITE_BYTE:
					   writeByteDIOPort();
					   break;
				case WRITE_BIT:
					  writeBitDIOPort();
					  break;

				case LOOP_BACK_TEST:
					DIOPortLoopbackTest();
					break;
				
				case 'q':
					return 0;
            		break;

				default: // invalid option selected
					printf("Please Enter valid option \r\n");
					break;
		}
	}while ( input_buffer[0] != 'q' );





	JNMMCANClose(can_fd);	
	
	
	printf ( "DSCJNMMCAN4LP-XT_DEMO DIO Application Closed\n" );
	return 0;
}
//=============================================================================
// Name: readByteDIOPort()
// Desc: Read a port A-B by receiving a byte from the selected port's  register.
// Parameters:void
// Return: void
//=============================================================================
void readByteDIOPort()
{
	int config;
	int port = 0;
	int input_byte = 0; 
	char input_buffer[20];

	

	printf ("Enter port number (0-1):");
	fgets ( input_buffer, 20, stdin );
   	sscanf ( input_buffer, "%d", &port );
	
	config =0;
	JNMMGPIOConfig(port,config);

	printf("\nPress ENTER key to stop reading ...\r\n");

	do
	{
		JNMMGPIORead ( port,&input_byte);
		
		
		dscsleep(1000);

		printf("Byte value received from port %d = 0x%x\r\n", port, input_byte);

	}
	while ( !( kbhit() ) );
	
	return;

}
//=============================================================================
// Name: readByteAllDIOPorts()
// Desc: Read a BYTE value from all the  ports A-B .
// Parameters:void
// Return: void
//=============================================================================
void readByteAllDIOPorts()
{

	int config;
	int port = 0;
	int input_byte = 0; 
	char input_buffer[20];
	
	for(port=0;port<2;port++)
	{
		config =0;
		JNMMGPIOConfig(port,config);
	}

	do
	{
		printf("Port:\t0\t1\n");
		printf("Value:\t");
		{
			JNMMGPIORead ( port,&input_byte);
			
			printf("0x%X\t",input_byte);

		}
		dscsleep(1000);
		printf("\n\n");

	}while ( !( kbhit() ) );

}

//=============================================================================
// Name: readAllBitsDIOPort()
// Desc: Read all bits  from the selected port's  register.
// Parameters:void
// Return: void
//=============================================================================
void  readAllBitsDIOPort()
{
	int config;
	int port = 0;
	int input_byte = 0; 
	char input_buffer[20];
	int digital_value = 0; // digital representation of the bit
	int intBuff = 0; // temp variables of size int
	BYTE bit = 0;
	for(port=0;port<2;port++)
	{
		config =0;
		JNMMGPIOConfig(port,config);
	}
	printf ("Enter port number (0-1):");
	fgets ( input_buffer, 20, stdin );
    sscanf ( input_buffer, "%d", &port );

	printf("\nPress ENTER key to stop reading ...\r\n");

	do
	{
		printf("The port %d each bit values are \n",port);
		printf("--------------------------------------------------------------------\r\n");
		printf("BitNo:\t7\t6\t5\t4\t3\t2\t1\t0\r\n");
		printf("--------------------------------------------------------------------\r\n");
		printf("Value:");
		
		for(intBuff=7 ;intBuff>=0;intBuff--)
		{
			if(port == 0)
			{
				bit = (BYTE) intBuff;
			}
			else if(port == 1)
			{
				bit = 8 + (BYTE) intBuff;
				
			}
			
		//printf("before digital value before %x\n",digital_value);	
		JNMMGPIOReadBit ( port,bit, &digital_value );

		printf("\t%x",digital_value);
		//printf("after digital value %x\n",digital_value);
		}

		printf("\n");
		dscsleep(1000);

	}while ( !( kbhit() ) );

	return;

}

//=============================================================================
// Name: writeByteDIOPort()
// Desc: Program a port A-C by sending a byte to the selected port's
//	  register.
// Parameters: void
// Return: void
//=============================================================================
void writeByteDIOPort()
{
	int config;
	int port = 0;
	char input_buffer[20];
	BYTE output_byte = 0;
	int intBuff = 0;
#if 0
	for(port=0;port<2;port++)
	{
		config=0;
		JNMMGPIOConfig(port,config);
	}
#endif
	printf ("Enter port number (0-1):");
	fgets ( input_buffer, 20, stdin );
   	sscanf ( input_buffer, "%d", &port );

	config =0x1;
	JNMMGPIOConfig(port,config);

	do
	{
		printf("Enter value 0-255 or q to quit:");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &intBuff );
		
		if ( input_buffer[0] == 'q' )
		{
			intBuff = 'q';
		}
		else
		{
			output_byte = (BYTE)intBuff;
			
			JNMMGPIOWrite(port,output_byte);
			
			printf("The Byte value  %d is sent to port %d \n",output_byte,port);
		}
	}while ( input_buffer[0] != 'q' );

	return;
}

//=============================================================================
// Name: writeBitDIOPort()
// Desc: Program a port A-C by sending a digital value to the selected
// 	  port's bit register.
// Parameters:void
// Return: void
//=============================================================================
void writeBitDIOPort()
{
	int config;
	BYTE digital_val = 0;
	BYTE bit = 0;
	int port = 0;
	int intBuf = 0;
	char input_buffer[20];

	for(port=0;port<2;port++)
	{
		config=0;
		JNMMGPIOConfig(port,config);
	}
	printf ("Enter port number (0-1):");
	fgets ( input_buffer, 20, stdin );
    sscanf ( input_buffer, "%d", &port );
	
	config = 0x1;
	JNMMGPIOConfig(port,config);

	do
	{
		
		printf ("Enter Bit (0-7):or q to quit :");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &intBuf );

		if(port == 0)
		{
			bit = intBuf;
		}
		else if(port == 1)
		{
			bit = 8 + intBuf;
		}
		
		if(input_buffer[0] != 'q')
		{
			printf ("Enter Bit value (0-1):");
			fgets ( input_buffer, 20, stdin );
			sscanf ( input_buffer, "%d", &intBuf );
			digital_val = intBuf;


			JNMMGPIOWriteBit(port,bit,digital_val);
			
		}
	}while( input_buffer[0] != 'q' );

return;
}
//=============================================================================
// Name: DIOPortLoopbackTest()
// Desc: loop back test on various ports
// Parameters: void
// Return: void
//=============================================================================
void DIOPortLoopbackTest()
{
	int config;
	BYTE digital_val = 0;
	BYTE bit = 0;
	int port = 0;
	int intBuff = 0;
	char input_buffer[20];
	int output_port = 0,input_port = 0;
	BYTE output_byte = 0;
	int input_byte = 0;
	int errorcount=0;
	

	for(port=0;port<2;port++)
	{
		config=0;
		JNMMGPIOConfig(port,config);
	}

	do
	{
		printf ("Enter port number for input (0-1) or  Enter -1 to go to Main menu:");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &input_port );
		if(input_port==-1) return;
		

		printf ("Enter port number for output (0-1):");
		fgets ( input_buffer, 20, stdin );
		sscanf ( input_buffer, "%d", &output_port );
		if(output_port==-1) return;
			
		if (( input_port > 1 || input_port < 0 ) || (output_port > 1 || output_port < 0 ) )
		{
			printf ( "ERROR: Valid Port Numbers are: \n ");
			return;
		}

		config = 0;
		JNMMGPIOConfig(input_port,config);
	
		config = 1;
		JNMMGPIOConfig(output_port,config);

		errorcount=0;
		for(intBuff=0; intBuff<=255;intBuff++)
		{  
		  output_byte=(BYTE) intBuff;
			
			JNMMGPIOWrite( output_port, output_byte );
			
			
			JNMMGPIORead(  input_port,&input_byte) ;
			
			 
			if(output_byte !=input_byte)
			{
				  printf("Error: Output: 0x%x  Input: 0x%x \n",output_byte,input_byte);
					  errorcount+=1;
			}
		}
		printf("\nValues written : %d and Errors: %d \n",intBuff,errorcount);
	}while(1);


}
