
// Structure and Function prototype

/* CAN device statistics  */
typedef struct can_device_stats {
unsigned int txerr;    /* Tx errors */
unsigned int rxerr;    /* Rx errors */
unsigned int txpackets; /* Tx transmitted count */
unsigned int rxpackets;  /* Rx transmitted count */
unsigned int rx_over_errors; /* Rx over error count */
unsigned int error_active;  /* Error active enable */
unsigned int stat_change;  /*stat changed from passive to active ,vice versa */
unsigned int bit_error; /* bit error  */
unsigned int stuff_error; /* stuff error  */
unsigned int form_error; /* Form error  */
unsigned int bus_error;        /* Bus errors */
unsigned int bus_error_enable; /* bit error enable */
unsigned int error_warning;    /* Changes to error warning state */
unsigned int error_passive;    /* Changes to error passive state */
unsigned int bus_off;          /* Changes to bus off state */
unsigned int arbitration_lost; /* Arbitration lost errors */
unsigned int arbitration_lost_enable; /* arbitration lost error enable  */
}can_device_stats,*pcan_device_stats;


/* CAN config structure */
typedef struct _CANCONFIG
{
	unsigned int Can_ch;
	unsigned int BaudRate;
}CANCONFIG,*PCANCONFIG;


/* CAN Message information strcuture */
typedef struct _CANMESSAGE
{
	unsigned int Can_ch;
	unsigned int Rtr;
	unsigned int Msg_type;
	unsigned int Tb_Type;
	unsigned int Msg_ID;
	unsigned int Msg_len;
	unsigned char Data[68];
}CANMESSAGE,*PCANMESSAGE;

/* CAN port configuration information for PORTA/PORTB*/
typedef struct _PORT_CONFIG
{
	unsigned int Port;
	unsigned int Control;
	unsigned char data;
}PORTCONFIG,*PPORTCONFIG;

/* CAN BaseAddr structure */
typedef struct _CANADDR
{
	unsigned int base_addr;
	unsigned int DIO_addr;
	
}CANADDR,*PCANADDR;


/* CAN strcuture for read/write the register from the driver to application vice versa */
typedef struct _CANRW
{
	unsigned int Can_ch;
	unsigned char offset;
	unsigned char data;
	
}CANRW,*PCANRW;

struct pci_dev *pci_dev;
#define MAXQ 100

typedef struct can{
	void *base_addr;
	int device_cnt ;
	int int_level;
	struct can_device_stats can_stats[4];
	unsigned int Front0;
	unsigned int Rear0;
	unsigned char Q0[MAXQ+1][20];
	unsigned int Front1;
	unsigned int Rear1;
	unsigned char Q1[MAXQ+1][20];
	unsigned int Front2;
	unsigned int Rear2;
	unsigned char Q2[MAXQ+1][20];
	unsigned int Front3;
	unsigned int Rear3;
	unsigned char Q3[MAXQ+1][20];
	wait_queue_head_t read_queue0;
	wait_queue_head_t write_queue0;
	wait_queue_head_t write_queue1;
	wait_queue_head_t write_queue2;
	wait_queue_head_t write_queue3;
	struct mutex tx0;
	struct mutex tx1;
	struct mutex tx2;
	struct mutex tx3;
}can_dev;

void Datareceive(int can,can_dev *candev_p);

static irqreturn_t JNMM_PCI_Handler(int irq, void *dev_id);

// CAN index
#define CAN_0 0
#define CAN_1 1
#define CAN_2 2
#define CAN_3 3
#define CAN_INVALID 4

// ISA mode

#define ISA_MODE 1

#define SET_VALUE 1

/* IOCTL Control Codes */

#define IOCTL_BAUDRATE 0x12

#define IOCTL_RX_0       0x13

#define IOCTL_DIO_CONFIG 0x14

#define IOCTL_DIO_READA 0x15

#define IOCTL_DIO_READB 0x16

#define IOCTL_RX_1      0x17

#define IOCTL_RX_2	0x18

#define IOCTL_RX_3      0x19

#define IOCTL_DEV_STATS_0 0x20

#define IOCTL_DEV_STATS_1 0x21

#define IOCTL_DEV_STATS_2 0x22

#define IOCTL_DEV_STATS_3 0x23

#define IOCTL_DEV_READ 0x24

#define IOCTL_DEV_WRITE 0x25

#define IOCTL_DEV_BASE_ADDR  0x26

#define IOCTL_DEV_DIO_BASE 0x27

#define IOCTL_TX_0 0x28

#define IOCTL_TX_1 0x29

#define IOCTL_TX_2 0x30

#define IOCTL_TX_3 0x31

// Interrupt Status 

#define DIO_INTR 0x03

#define RIF_STAT 0x80

#define ROIF_STAT 0x40

#define RFIF_STAT 0x20

#define RAFIF_STAT 0x10

#define TPIF_STAT 0x08

#define TSIF_STAT 0x04

#define EIF_STAT 0x02

#define AIF_STAT 0x01
// Device Registers

#define RBUF              0x00       /* Receive Buffer Data                   */
#define TBUF              0x48       /* Receive Buffer Data                   */
#define CFG_STAT          0x90       /* Ctrl/Status register                   */
#define TCMD              0x91       /* Transmit command register              */
#define TCTRL             0x92       /* Transmit control register              */
#define RCTRL             0x93       /* Rx command register                    */
#define RTIE              0x94       /* Rx-Tx interrupt enable register        */
#define RTIF              0x95       /* Rx-Tx interrupt Flag register          */
#define ERRINT            0x96       /*ERRor INTerrupt Enable and Flag Register*/
#define BITTIME0          0x98       /*Bit Timing Register                     */
#define BITTIME1          0x99       /*Bit Timing Register                     */
#define BITTIME2          0x9a       /*Bit Timing Register                     */
#define SPRESC            0x9c       /*Prescaler Register                      */
#define FPRESC            0x9d       /*Prescaler Register                      */
#define LIMIT             0x9e       /*Warning Limits Register                 */
#define EALCAP            0x9f       /*Error,Arbitration Lost Capture Register */
#define RECNT             0xa0       /*Error Counter Register                  */
#define TECNT             0xa1       /*Error Counter Register                  */
#define ACFCTRL           0xa2       /*Acceptance Filter Control Register      */
#define ACF_3             0xa4       /*Acceptance code                         */
#define ACF_2             0xa5       /*Acceptance code                         */
#define ACF_1             0xa6       /*Acceptance code                         */
#define ACF_0             0xa7       /*Acceptance code                         */
#define ACF_EN_0          0xa8       /*Acceptance Filter Enable                */
#define ACF_EN_1          0xa9       /*Acceptance Filter Enable                */
#define RBUF_COUNT        0xAA       /*No of Receive buffer Active count       */

/* TBUF Offsets */
#define MSG_ID_1          0x0
#define MSG_ID_2          0x1
#define MSG_ID_3          0x2
#define MSG_ID_4          0x3
#define TBUF_CTRL         0x4       /*Transmit Frame information register     */
#define TBUF_DATA         0x8       /*Transmit Frame information register     */
#define RBUF_DATA         0x8       /*Transmit Frame information register     */

/* Ctrl/Status register                   */

#define BIT_0            (1 << 0)
#define BIT_1            (1 << 1)
#define BIT_2            (1 << 2)
#define BIT_3            (1 << 3)
#define BIT_4            (1 << 4)
#define BIT_5            (1 << 5)
#define BIT_6            (1 << 6)
#define BIT_7            (1 << 7)       


/* DIO CONFIG Offsets */
#define	DIO_PORT_A		0
#define	DIO_PORT_B		1
#define	DIO_DIRECTION_CTRL	2
#define	FPGA_MAJOR_VER		0
#define	FPGA_MINOR_VER		1
#define	BOARD_MAJOR_VER		2
#define	BOARD_MINOR_VER		3





