#ifndef CAN_H
#define CAN_H

#ifdef __cplusplus
extern "C" {
#endif


#define CAN_SPEED_1M		0
#define CAN_SPEED_800K		1
#define CAN_SPEED_500K		2
#define CAN_SPEED_250K		3
#define CAN_SPEED_125K		4
#define CAN_SPEED_100K		5
#define CAN_SPEED_50K		6
#define CAN_SPEED_20K		7
#define CAN_SPEED_10K		8


#define MSG_STANDARD		0
#define MSG_EXTENDED		1

#define MAX_DEV_SUPPORT         4


#define PORT_A 0

#define PORT_B 1

#define HIGH_VALUE 1

#define LOW_VALUE  0

#define BYTE unsigned char

#define UNSIGNED_BYTE_MAX 255


#define DIO_PORT_A  "/sys/kernel/dsc_can/dioA"
#define DIO_PORT_B  "/sys/kernel/dsc_can/dioB"

#define FPGA_VERSION  "/sys/kernel/dsc_can/fpga_ver"
#define BOARD_VERSION  "/sys/kernel/dsc_can/board_ver"



#define FPGA_ID         1
#define BOARD_ID        2

static int pci_mode =0;

static int isa_mode =1;

/* CAN device statistics  */
typedef struct can_device_stats {
unsigned int txerr;    /* Tx errors */
unsigned int rxerr;    /* Rx errors */
unsigned int txpackets; /* Tx transmitted count */
unsigned int rxpackets;  /* Rx transmitted count */
unsigned int rx_over_errors; /* Rx over error count */
unsigned int error_active;  /* Error active enable */
unsigned int stat_change;  /*stat changed from passive to active ,vice versa */
unsigned int bit_error; /* bit error  */
unsigned int stuff_error; /* stuff error  */
unsigned int form_error; /* Form error  */
unsigned int bus_error;        /* Bus errors */
unsigned int bus_error_enable; /* bit error enable */
unsigned int error_warning;    /* Changes to error warning state */
unsigned int error_passive;    /* Changes to error passive state */
unsigned int bus_off;          /* Changes to bus off state */
unsigned int arbitration_lost; /* Arbitration lost errors */
unsigned int arbitration_lost_enable; /* arbitration lost error enable  */
}can_device_stats;

/* CAN Message information strcuture */
typedef struct _CANMESSAGE
{
	unsigned int Can_ch;
	unsigned int Rtr;
	unsigned int Msg_type;
	unsigned int Tb_Type; 
	unsigned int Msg_ID;
	unsigned int Msg_len;
	unsigned char Data[68];
}CANMESSAGE,*PCANMESSAGE;

/* CAN port configuration information for PORTA/PORTB*/
typedef struct _PORT_CONFIG
{
	unsigned int Port;
	unsigned int Control;
	unsigned char data;
}PORTCONFIG,*PPORTCONFIG;

/* CAN config structure */
typedef struct _CANCONFIG
{
	unsigned int Can_ch;
	unsigned int BaudRate;
	
}CANCONFIG,*PCANCONFIG;

/* CAN BaseAddr structure */
typedef struct _CANADDR
{
	unsigned int base_addr;
	unsigned int DIO_addr;
	
}CANADDR,*PCANADDR;


/* CAN strcuture for read/write the register from the driver to application vice versa */
typedef struct _CANRW
{
	unsigned int Can_ch;
	unsigned char offset;
	unsigned char data;
	
}CANRW,*PCANRW;


/* IOCTL Control Codes */


#define IOCTL_BAUDRATE 0x12

#define IOCTL_RX_0       0x13

#define IOCTL_DIO_CONFIG 0x14

#define IOCTL_DIO_READA 0x15

#define IOCTL_DIO_READB 0x16

#define IOCTL_RX_1      0x17

#define IOCTL_RX_2	0x18

#define IOCTL_RX_3      0x19

#define IOCTL_DEV_STATS_0 0x20

#define IOCTL_DEV_STATS_1 0x21

#define IOCTL_DEV_STATS_2 0x22

#define IOCTL_DEV_STATS_3 0x23

#define IOCTL_DEV_READ 0x24

#define IOCTL_DEV_WRITE 0x25

#define IOCTL_DEV_BASE_ADDR  0x26

#define IOCTL_DEV_DIO_BASE 0x27

#define IOCTL_TX_0 0x28

#define IOCTL_TX_1 0x29

#define IOCTL_TX_2 0x30

#define IOCTL_TX_3 0x31

/* User defined functions*/
int JNMMCANOpen(int * flag);

void JNMMCANClose(int can_ch);

int JNMMCANConfig(int can_ch,  CANCONFIG CanParam);

int JNMMCANTx( int can_ch, CANMESSAGE* msg);

int JNMMCANRx(int can_ch, CANMESSAGE* msg);

int JNMMCANErrorStat( int can_ch,CANCONFIG CanParam, can_device_stats* stats);

int JNMMCANDIOAddrInit(unsigned int dio_addr);

int JNMMCANBaseAddrInit(unsigned int base_addr);

unsigned short JNMMGetFPGAVersion();

unsigned short JNMMGetBoardVersion();

int JNMMCANRead(int can_ch,unsigned char offset);

int JNMMCANWrite(int can_ch,unsigned char offset,unsigned char data);

//Digital I/O functions

unsigned char JNMMGPIOConfig( int Port, int Config);

unsigned char JNMMGPIORead( int Port,int *data);

unsigned char JNMMGPIOWrite( int Port, int data);

unsigned char JNMMGPIOWriteBit( int Port,int Bit, int data);

unsigned char JNMMGPIOReadBit( int Port,int Bit,int * data);





#ifdef __cplusplus
} /* Closes the extern "C" */
#endif
#endif
